function onOpen(e) {
  var ui = DocumentApp.getUi();
  // Or DocumentApp or FormApp.
  ui.createMenu('Dialogue Editor')
      .addItem('Format Text', 'formatText')
      .addToUi();
}

function refresh(e)
{
  formatText();
}

function formatText() {
  var body = DocumentApp.getActiveDocument().getBody();
  formatEverything(body);
  formatTitles(body);
  formatActorNames(body);
  formatBranches(body);
  formatSequences(body);  
  formatConditions(body);
  formatScripts(body);
}

function formatEverything(body)
{  
 body.editAsText().setUnderline(false);
 body.editAsText().setBold(true).setForegroundColor("#4d004d").setFontSize(10);
}


function formatTitles(body)
{
  var regEx = "(#).*";
  var result = body.findText(regEx);
   
  while (result != null) {
     
    var startOffset = result.getStartOffset();
    var endOffset = result.getEndOffsetInclusive();
      
    var text = result.getElement().asText();
    text.setUnderline(startOffset, endOffset, true);
    text.setForegroundColor("#a25a1a");
    
    result = body.findText(regEx, result);
  }
  
}

function formatActorNames(body)
{
  var regEx = ".*(:)";
  var result = body.findText(regEx);
  var colors = ["#009999","#c35300","#002cca","#5dbb00","#8500af","#bcac00"];
  var actorColors = {};
  var colorIndex = 0;

  while (result != null) {
     
    var startOffset = result.getStartOffset();
    var endOffset = result.getEndOffsetInclusive();
      
    var text = result.getElement().asText();
    var characterName = text.getText();
    characterName = characterName.substring(0, characterName.indexOf(':'));
    Logger.log(characterName);

    
    if(!actorColors.hasOwnProperty(characterName))
    {
      actorColors[characterName] = colors[colorIndex];
      colorIndex ++;
      if(colorIndex >= colors.length) colorIndex = 0;
    }
    
    text.setForegroundColor(actorColors[characterName]);
    text.setBold(startOffset, endOffset, false);
    
    result = body.findText(regEx, result);
  }
  
  
}

function formatBranches(body)
{
  var regEx = "\\[.{0,}\\]";
  var result = body.findText(regEx);
   
  while (result != null) {
     
    var startOffset = result.getStartOffset();
    var endOffset = result.getEndOffsetInclusive();
      
    var text = result.getElement().asText();
    text.setForegroundColor(startOffset, endOffset, "#6D4C41");
    text.setBold(startOffset, endOffset, false);
    
    result = body.findText(regEx, result);
  }
}

function formatSequences(body)
{
  var regEx = "({).*(})";
  var result = body.findText(regEx);
  
  while (result != null) {
     
    var startOffset = result.getStartOffset();
    var endOffset = result.getEndOffsetInclusive();
      
    var text = result.getElement().asText();
    text.setBold(startOffset, endOffset, false);
    text.setForegroundColor(startOffset, endOffset, "#bebebe");
    
    result = body.findText(regEx, result);
  }
}

function formatConditions(body)
{
  var regEx = "(\\|).*(\\|)";
  var result = body.findText(regEx);
  
  while (result != null) {
     
    var startOffset = result.getStartOffset();
    var endOffset = result.getEndOffsetInclusive();
      
    var text = result.getElement().asText();
    text.setBold(startOffset, endOffset, false);
    text.setForegroundColor(startOffset, endOffset, "#868C8A");
    
    result = body.findText(regEx, result);
  }
}

function formatScripts(body)
{
  var regEx = "(±).*(±)";
  var result = body.findText(regEx);
  
  while (result != null) {
     
    var startOffset = result.getStartOffset();
    var endOffset = result.getEndOffsetInclusive();
      
    var text = result.getElement().asText();
    text.setBold(startOffset, endOffset, false);
    text.setForegroundColor(startOffset, endOffset, "#868C8A");
    
    result = body.findText(regEx, result);
  }
}